/// <summary>
/// C# XBGM# Xbox Game Manager.
///  
/// Copyright (C) 2004  XBGM# Team (See AUTHORS)
/// 
/// 
/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.
/// 
/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.
/// 
/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
/// 
/// Bug fixes, suggestions and comments should be sent to xbgm@kayaweed.net
/// 
/// Change Log:
/// 
/// Revision 0.1  2005-03-19 Pascal Giard <evilynux@gmail.com>
/// - initial version
///
/// </summary>

// created on 2005-03-19 at 16:50

using EnterpriseDT.Net.Ftp;
using System;
using System.Collections;
using System.IO;
using System.Net;
using System.Net.Sockets;

namespace Xbgm.Core
{
	public delegate void Notifier(string[] rootspace);
	public delegate void Logger(string msg);
	public delegate void UpdateStatus(bool online);
	
	public class FtpToXbox
	{	
		// Event send to the Main GUI
		public event EventHandler EventUpdated;
		public event Notifier notifyViews;
		public event Logger loggerViews;
		public event UpdateStatus StatusViews;
		public Action action = Action.ready;
		public Dashboards dashboard = Dashboards.unknown;
		public Parser.Summary gameSummary;
		public string[] Rootspace;
		public string Msg;
		public string MsgError;
		public bool In_Thread;
		public bool Online;
		public bool Done;
		public int Index = -1;
		
		public Xbgm.Config 			config;
		//private Xbgm.Configuration 	cfg;
		private Xbgm.Core.Messages 	msg;
		private Xbgm.Core.Parser 	parser;
		private Xbgm.Core.FtpLogger logger;
		
		private FTPClient ftp;
				
		// Constructor
		public FtpToXbox(ref Xbgm.Config config, Messages msg)
		{
			//this.cfg = cfg;
			//cfg = new Configuration();
			this.config = config;
			this.msg = msg;
			this.MsgError = null;
			this.Rootspace = null;
			this.parser = new Parser();
			this.logger = new FtpLogger();
			this.In_Thread = false;
			this.Online = false;
			this.Done = false;
			this.ftp = null;
		}
		 
		public enum Action
			{
				ready, scan, rename, delete, launch, reboot, shutdown,
				xbmc, image, freespace,	allfreespace, format, refresh, 
				error, connect
			};
		
		public enum Dashboards
			{
				avalaunch, xbmc, unleachx, unknown
			};		
		
		public bool InThread{
			get { return In_Thread; }
			set { In_Thread = value;}
		}
		
		public void ChangeLabelResults(string[] rootspace)
		{
			#if ( DEBUG )
				Console.WriteLine("In FtpToXbox::ChangeLabelResults\n");
			#endif
			if (In_Thread == false)
				notifyViews(rootspace);
			else {
				Rootspace = rootspace;
				if(this.EventUpdated != null)
					this.EventUpdated(null, null); 
			}
		}
		
		public void UpdateStatus(bool online)
		{
			#if ( DEBUG )
				Console.WriteLine("In FtpToXbox::UpdateStatus\n");
			#endif
			if (In_Thread == false)
				StatusViews(online);
			else {
				Online = online;
				if(this.EventUpdated != null)
					this.EventUpdated(null, null); 
			}
		}
		
		public void Log()
		{
			#if ( DEBUG )
				Console.WriteLine("In FtpToXbox::Log\n");
			#endif
			if (In_Thread == false)
				loggerViews(logger.logIt());
			else {
				Msg = logger.logIt();
				if(this.EventUpdated != null)
					this.EventUpdated(null, null); 
			}
			logger.Flush();
			logger.Close();
		}	
		
		private void Connect()
		{
			try
			{
				// Init the the write memory stream for ftp log
				logger.Init();
				// Update status bar to connecting...
				this.action = Action.connect;
				if(InThread & this.EventUpdated != null) // Tell main windows to wake up
					this.EventUpdated(null, null);
				
				Index = config.GetIndexActive();
				// Create socket
				#if ( DEBUG ) // Forcing logging to console in debug mod
				ftp = new FTPClient(config.xboxs[Index].ip, config.xboxs[Index].port);
				ftp.DebugResponses(true);
				#else
				if (config.ftpverbose) {
					ftp = new FTPClient(config.xboxs[Index].ip, config.xboxs[Index].port, logger.logWriter, 0);
					ftp.DebugResponses(true);
				}
				else
					ftp = new FTPClient(config.xboxs[Index].ip, config.xboxs[Index].port);					
				#endif

				// With logger
				//ftp = new FTPClient(cfg.getFtpInfo().getIp(), cfg.getFtpInfo().getPort(), logger.logWriter, 0);
				// Standar
				//ftp = new FTPClient(cfg.getFtpInfo().getIp(), cfg.getFtpInfo().getPort());

				// Login
				//ftp.Login(cfg.getFtpInfo().getUser(), cfg.getFtpInfo().getPassword());
				ftp.Login(config.xboxs[Index].username, config.xboxs[Index].password);
				ftp.System();

				UpdateStatus(true);			
			}
			catch (Exception ex)
			{
				#if (DEBUG)
					Console.WriteLine("Caught exception (in FtpToXbox::Connect): " + ex.Message);
				#endif
				if (In_Thread) { // in thread mode
					MsgError = ex.Message;
					// Set To error mode
					this.action = Action.error;
					// To refresh the statusbar and stop thread;
					if(this.EventUpdated != null)
						this.EventUpdated(null, null); 
				} else {
					this.action = Action.ready;
					// Make sure methods get to know it.
					throw new Exception("Can't connect to Xbox:\n" + ex.Message);
				}
			} 
		}
		
		private void Disconnect()
		{
			// The default quit wait for a reply but there is none
			// so it crash, need to catch exception
			// or do it like this, I made my own Quit with out reply	
			try
			{
				ftp.QuitXbox();
				UpdateStatus(false);
				Log();
				this.action = Action.ready;
				ftp = null;
			}
			catch (FTPException ex)
			{
				#if (DEBUG)
					Console.WriteLine("Caught exception (in FtpToXbox::Disconnect): " + ex.Message);
				#endif
				if (In_Thread) { // in thread mode
					MsgError = ex.Message;
					// Set To error mode
					this.action = Action.error;
					// To refresh the statusbar and stop thread;
					if(this.EventUpdated != null)
						this.EventUpdated(null, null); 
				} else {
					this.action = Action.ready;
					// Make sure methods get to know it.
					throw new Exception("Error Disconnecting:\n" + ex.Message);
				}
			}
			System.GC.Collect();
		}
		

		public bool DeleteGame(ArrayList game, bool deleteSaveGames)
		{
			try
			{
				Connect();
			}
			catch (Exception ex)
			{
				#if (DEBUG)
					Console.WriteLine(ex.Message);
				#endif
				return false;
			}

			try 
			{
				ftp.TransferType = FTPTransferType.BINARY;
				
				// I don't know why but they do it
				ftp.Site("FREEROOTSPACEDISABLE ");			
				
											// xbe full path
				bool ok = ftp.Site("DELTREE " + game[2]);			
				if(ok == false)
				{
					// then we do it the old way
					ftp.Rmdir(game[2].ToString());
				}
				
				if(deleteSaveGames)
				{										//ID
					ok = ftp.Site(@"DELTREE /E:/UDATA/" + game[3]);
					ok = ftp.Site(@"DELTREE /E:/TDATA/" + game[3]);
				}
			}
			catch (FTPException ex)
			{
				Console.WriteLine("Caught FTP exception: " + ex.Message);
				this.action = Action.ready;
			}
			
			Disconnect();
			
			return true;
		}
		
		public bool RenameGame(ArrayList game, string newGameName)
		{
			try
			{
				Connect();
			}
			catch (Exception ex)
			{
				#if (DEBUG)
					Console.WriteLine(ex.Message);
				#endif
				return false;
			}
			ftp.TransferType = FTPTransferType.BINARY;
			ftp.Site("XBERENAME \"" + game[2] + game[1] + "\" " + newGameName);			
			Disconnect();
			return true;
		}
		
		public bool LaunchGame(ArrayList game)
		{
			try
			{
				Connect();
			}
			catch (Exception ex)
			{
				#if (DEBUG)
					Console.WriteLine(ex.Message);
				#endif
				return false;
			}
			ftp.TransferType = FTPTransferType.BINARY;
			ftp.Site("LAUNCH " + game[2] + game[1]);			
			Disconnect();
			return true;
		}
		
		public void FormatDrive(string drive)
		{
			Connect();
			#if ( DEBUG )
				Console.WriteLine("Format drive " + drive);
			#endif
			// DriveSelected string
			ftp.Site("FORMAT " + drive);

			Disconnect();
		}
		
		public void ActionNotOnGame(Action xboxAction)
		{
			try
			{
				In_Thread = false;
				Connect();
	
				
				this.action = xboxAction;
				
				switch(this.action) 
				{
					case Action.shutdown: // Shutdown
					{
						ftp.Site("SHUTDOWN");
						break;
					}
	
					case Action.reboot: // Restart
					{
						ftp.Site("REBOOT");
						break;
					}
	
					case Action.xbmc: // Launch XBMC
					{
						ftp.TransferType = FTPTransferType.BINARY;
						if (this.dashboard == Dashboards.avalaunch || 
						   this.dashboard == Dashboards.unleachx)
							ftp.Site("LAUNCH " + config.xboxs[Index].xbmc);	// XBMC xbe full path
						else if (this.dashboard == Dashboards.xbmc)
							ftp.Site("RunXBE " + config.xboxs[Index].xbmc);			
						break;
					}
						
					case Action.refresh: // Refresh Avalaunch menu
					{
						ftp.TransferType = FTPTransferType.BINARY;
		
						// Refresh Avalaunch menu
						ftp.Site("REFRESH");			
						break;
					}
						
					default:            
	           			Console.WriteLine("Invalid Action, has to be shutdown, reboot , refresh or xbmc.");            
	           			break;  
	
				}
				Disconnect();
			}
			catch (Exception ex)
			{
				#if (DEBUG)
					Console.WriteLine("Caught exception (in FtpToXbox::ActionNotOnGame): " + ex.Message);
				#endif
				if (ftp != null)
					Disconnect();
				throw ex;
			}		
		}

		public string GetTitlesCached()
		{
			string freespace = "N/A";
			string[] dir = config.GetDirectoryActive();
			parser.dir = dir[0];
			this.gameSummary = parser.TitlesCached();
			this.action = Action.ready;
			
			return freespace;			
		} // End of GetTitlesCached
		
		public string GetTitles(bool getIcons)
		{
			string freespace = "N/A";
			
			try
			{
				Connect();
			}
			catch (Exception ex)
			{
				#if (DEBUG)
					Console.WriteLine("Caught exception (in FtpToXbox::GetTitles):" + ex.Message);
				#endif
				Disconnect();
				throw new Exception(ex.Message);
			}
			
			this.action = Action.scan;
			if(InThread & this.EventUpdated != null) // Tell main windows to wake up
				this.EventUpdated(null, null);
			ftp.TransferType = FTPTransferType.BINARY;
			// Do all command and get titles game in a string[]
			// The problem of this is than i need to wait to recevied the all list of Game
			string[] dir = config.GetDirectoryActive();	
			for (int i = 0; i < dir.Length; i++) {
			try
			{
				#if (DEBUG)
				Console.WriteLine("gamedir: [" + dir[i] + "]");
				#endif
				//string onedir =  dir[0];
				parser.dir = dir[i];
				//Console.ReadLine();
				if (i == 0)
					gameSummary = parser.Titles(ftp.GetTitles(dir[i]), true);
				else {
					this.action = Action.scan; // We are doing it again for an another directory
					if(InThread & this.EventUpdated != null) // Tell main windows to wake up
						this.EventUpdated(null, null);
					gameSummary = parser.Titles(ftp.GetTitles(dir[i]), false);
				}

				//gameSummary = parser.Titles(ftp.GetTitles(cfg.getFtpInfo().getDirectory()));
				//Console.ReadLine();
				// Check if i need to DL icons
				if (getIcons)
				{
					this.action = Action.image;
					if(InThread & this.EventUpdated != null) // Tell main windows to wake up
						this.EventUpdated(null, null); 
					ftp.Site("TITLEIMAGES");
					ftp.Get(@"titleimage.zip", @"/Z:/titleimage.zip");
					Zip.Extract(@"titleimage.zip");
				}
			
			/* Unused LabelFreeAllResult has been supress
			 * 	this.action = Action.freespace;
				if(InThread & this.EventUpdated != null) // Tell main windows to wake up
					this.EventUpdated(null, null);
				// List free space Game Drive
				ftp.Site("FREEROOTSPACEENABLE");
				string[] rootspace = ftp.Dir("/", true);
				foreach(string space in rootspace)
				{
					//Console.WriteLine("Space:" + space + "]");
					string[] free = space.Split(' ');
					Console.WriteLine("[" + free[24].ToString() +"] == [" + dir[i].Substring(0, 2) +"]");
					if(free[24].Equals(dir[i].Substring(0, 2)))
					{
						#if (DEBUG)
							Console.WriteLine("Free Space:" + free[26].ToString());
						#endif
						freespace = free[26].ToString() + " MB";
					}
				}
				if (i != 0) { freespace = "N/A"; } // Multiple directory can't know how many space
				rootspace = null;
				*/
				}
				catch (Exception ex)
				{
					#if (DEBUG)
						Console.WriteLine("Caught exception (in FtpToXbox::GetTitles):" + ex.Message);
					#endif
					this.action = Action.error;
					MsgError = "No Game in directory " + dir[i];
					if(InThread & this.EventUpdated != null) // Tell main windows to wake up
						this.EventUpdated(null, null);
					Disconnect();
					throw new Exception(ex.Message +  "\nNo Game in directory " + dir[i]);
				}
			}// End for each directoy in config
			
			this.action = Action.allfreespace;
			// List free space for all Drive
			if (ftp.Site("FREEROOTSPACEENABLE") == true)
			{
				string[] rootspace = ftp.Dir("/", true);
				// Updating Views
				this.ChangeLabelResults(rootspace);
				rootspace = null;
				ftp.Site("FREEROOTSPACEDISABLE");
			}
			
			Disconnect();
			if(InThread & this.EventUpdated != null) // Tell main windows to wake up
					this.EventUpdated(null, null);
			return freespace;
		} // End of GetTitles
		
		public bool GetFreeSpace(string xboxDir, UInt32 size)
		{
			bool freespace = false;
			
			Connect();
			
			ftp.TransferType = FTPTransferType.BINARY;

			if (ftp.Site("FREEROOTSPACEENABLE") == true)
			{
				string[] rootspace = ftp.Dir("/", true);
				foreach(string space in rootspace)
				{
					//Console.WriteLine("Space:" + space + "]");
					string[] free = space.Split(' ');
					//Console.WriteLine("[" + free[24].ToString() +"] == [" + Entryxboxdir.Text.Substring(0, 2) +"]");
					if(free[24].Equals(xboxDir))
					{
						//Console.WriteLine("we need:" + fspace[0] + " and we have:" + free[26]);
						if (size > Convert.ToUInt32(free[26]))
							msg.Info("You don't enough free space on " + xboxDir);
						else
							freespace = true;
					}
					free = null;
				}
				rootspace = null;
				ftp.Site("FREEROOTSPACEDISABLE");	
			}
			else
			{ // Not using Avalaunch
				#if ( DEBUG )
					Console.WriteLine("Not using Avalaunch");
				#endif
				freespace = true;		
			}
			
			Disconnect();
			return freespace;
		}

		public bool CheckIfAvalaunch()
		{
			try
			{
				Connect();
				if (ftp.Site("FREEROOTSPACEENABLE") == false) // Check for avalaunch
				{
					Console.WriteLine("This feature requires Avalaunch, or a newer version.");
					MsgError = "This feature requires Avalaunch";
					Disconnect();
					return false;
				}
				Disconnect();
				return true;
			}
			catch (Exception ex)
			{
				#if (DEBUG)
					Console.WriteLine("Caught exception (in FtpToXbox::CheckIfAvalaunch):" + ex.Message);
				#endif
				if (ftp != null)
					Disconnect();
				throw new Exception(ex.Message);
			}
		}

		public bool CheckDashboard()
		{
			try
			{
				Connect();
				string dash = ftp.System();
				if (dash.StartsWith("UNIX emulation by Avalaunch")) {
					this.dashboard = Dashboards.avalaunch;
					Msg = "Dashboard: Avalaunch";
				}
				else if(dash.StartsWith("UNIX emulated by FileZilla") || 
				       dash.StartsWith("UNIX emulated by XBMC")) {
					this.dashboard = Dashboards.xbmc;
					Msg = "Dashboard: XBMC";
				}
				else if(dash.StartsWith("UNIX emulated by UnleashX")) {
					this.dashboard = Dashboards.unleachx;
					Msg = "Dashboard: UnleashX";
				}
				else {
					this.dashboard = Dashboards.unknown;
					Msg = "Dashboard: Unknown";
				}
				Disconnect();
				if (this.dashboard == Dashboards.avalaunch)
					return true;
				else if (this.dashboard == Dashboards.unknown) {
					MsgError = "This feature requires Avalaunch\nYour dashboard cannot be detected.";
					return false;
				} else {
					MsgError = "This feature requires Avalaunch\nYour " + Msg + ".";
					return false;
				}
			}
			catch (Exception ex)
			{
				#if (DEBUG)
					Console.WriteLine("Caught exception (in FtpToXbox::CheckDashboard):" + ex.Message);
				#endif
				if (ftp != null)
					Disconnect();
				throw new Exception(ex.Message);
			}
		}
		
		public void GetAllfreeSpace()
		{
			try
			{
				Console.WriteLine("Caught");
				if (this.dashboard == Dashboards.unknown) {
					ArrayList lst = new ArrayList();
					ftp.FileList(lst);
				}
				else if (this.dashboard == Dashboards.unleachx) {
					
				}
			}
			catch (Exception ex)
			{
				#if (DEBUG)
					Console.WriteLine("Caught exception (in FtpToXbox::CheckDashboard):" + ex.Message);
				#endif
				if (ftp != null)
					Disconnect();
				throw new Exception(ex.Message);
			}
		}
		
	/*	public Xbgm.Configuration getConfiguration()
		{
			return this.cfg;
		}
		
		public void setConfiguration(Xbgm.Configuration cfg)
		{
			this.cfg = cfg;
		}
	*/
	}
}
